@@ -828,6 +828,36 @@ set_char_encoding (term, charmap, tag)
   return False;
 }
 
+static
+struct {
+	char * alias;
+	char * codepage;
+} cpaliases [] =
+{
+	{"CP819", "ISO-8859-1"},
+	{"CP912", "ISO-8859-2"},
+	{"CP913", "ISO-8859-3"},
+	{"CP914", "ISO-8859-4"},
+	{"CP915", "ISO-8859-5"},
+	{"CP1089", "ISO-8859-6"},
+	{"CP813", "ISO-8859-7"},
+	{"CP916", "ISO-8859-8"},
+	{"CP920", "ISO-8859-9"},
+	{"CP919", "ISO-8859-10"},
+	{"CP923", "ISO-8859-15"},
+	{"CP28591", "ISO-8859-1"},
+	{"CP28592", "ISO-8859-2"},
+	{"CP28593", "ISO-8859-3"},
+	{"CP28594", "ISO-8859-4"},
+	{"CP28595", "ISO-8859-5"},
+	{"CP28596", "ISO-8859-6"},
+	{"CP28597", "ISO-8859-7"},
+	{"CP28598", "ISO-8859-8"},
+	{"CP28599", "ISO-8859-9"},
+	{"CP28603", "ISO-8859-13"},
+	{"CP28605", "ISO-8859-15"},
+};
+
 /**
    Set terminal character code mapping table according to encoding tag.
    Return True on success, False if tag unknown.
@@ -837,6 +867,25 @@ set_term_encoding (charmap, tag)
   char * charmap;
   char tag;
 {
+  /* handle generic codepage notation (cygwin 1.7 / DOS support) */
+  if (strisprefix ("CP", charmap)) {
+	if (set_char_encoding (True, charmap, tag)) {
+		return True;
+	} else {
+		int i;
+		/* check DOS/Windows ISO-8859 codepage aliases */
+		for (i = 0; i < arrlen (cpaliases); i ++) {
+			if (streq (charmap, cpaliases [i].alias)) {
+				if (set_char_encoding (True, cpaliases [i].codepage, tag)) {
+					return True;
+				}
+			}
+		}
+		(void) set_char_encoding (True, "ASCII", ' ');
+		return False;
+	}
+  }
+
   return set_char_encoding (True, charmap, tag);
 }
 
