@@ -1,5 +1,5 @@
-extern struct encoding_table_entry windows_table [];
-extern unsigned int windows_table_len;
+extern struct encoding_table_entry cp1252_table [];
+extern unsigned int cp1252_table_len;
 extern struct encoding_table_entry cygwin_table [];
 extern unsigned int cygwin_table_len;
 extern struct encoding_table_entry latin_9_table [];
@@ -14,14 +14,20 @@ extern struct encoding_table_entry koi8_u_table [];
 extern unsigned int koi8_u_table_len;
 extern struct encoding_table_entry isocyril_table [];
 extern unsigned int isocyril_table_len;
-extern struct encoding_table_entry w_cyrill_table [];
-extern unsigned int w_cyrill_table_len;
+extern struct encoding_table_entry cp1251_table [];
+extern unsigned int cp1251_table_len;
 extern struct encoding_table_entry koi8_t_table [];
 extern unsigned int koi8_t_table_len;
 extern struct encoding_table_entry pt154_table [];
 extern unsigned int pt154_table_len;
 extern struct encoding_table_entry georg_ps_table [];
 extern unsigned int georg_ps_table_len;
+extern struct encoding_table_entry cp855_table [];
+extern unsigned int cp855_table_len;
+extern struct encoding_table_entry cp866_table [];
+extern unsigned int cp866_table_len;
+extern struct encoding_table_entry cp1125_table [];
+extern unsigned int cp1125_table_len;
 extern struct encoding_table_entry isogreek_table [];
 extern unsigned int isogreek_table_len;
 extern struct encoding_table_entry isoarab_table [];
@@ -30,14 +36,26 @@ extern struct encoding_table_entry arabic_table [];
 extern unsigned int arabic_table_len;
 extern struct encoding_table_entry isohebr_table [];
 extern unsigned int isohebr_table_len;
-extern struct encoding_table_entry hebrew_table [];
-extern unsigned int hebrew_table_len;
+extern struct encoding_table_entry cp1255_table [];
+extern unsigned int cp1255_table_len;
+extern struct encoding_table_entry cp1253_table [];
+extern unsigned int cp1253_table_len;
+extern struct encoding_table_entry cp737_table [];
+extern unsigned int cp737_table_len;
+extern struct encoding_table_entry cp869_table [];
+extern unsigned int cp869_table_len;
+extern struct encoding_table_entry cp1256_table [];
+extern unsigned int cp1256_table_len;
+extern struct encoding_table_entry cp864_table [];
+extern unsigned int cp864_table_len;
+extern struct encoding_table_entry cp862_table [];
+extern unsigned int cp862_table_len;
 extern struct encoding_table_entry m_roman_table [];
 extern unsigned int m_roman_table_len;
-extern struct encoding_table_entry pc437_table [];
-extern unsigned int pc437_table_len;
-extern struct encoding_table_entry pc850_table [];
-extern unsigned int pc850_table_len;
+extern struct encoding_table_entry cp437_table [];
+extern unsigned int cp437_table_len;
+extern struct encoding_table_entry cp850_table [];
+extern unsigned int cp850_table_len;
 extern struct encoding_table_entry latin_2_table [];
 extern unsigned int latin_2_table_len;
 extern struct encoding_table_entry latin_3_table [];
@@ -54,6 +72,30 @@ extern struct encoding_table_entry latin_8_table [];
 extern unsigned int latin_8_table_len;
 extern struct encoding_table_entry latin_10_table [];
 extern unsigned int latin_10_table_len;
+extern struct encoding_table_entry cp1250_table [];
+extern unsigned int cp1250_table_len;
+extern struct encoding_table_entry cp1254_table [];
+extern unsigned int cp1254_table_len;
+extern struct encoding_table_entry cp1257_table [];
+extern unsigned int cp1257_table_len;
+extern struct encoding_table_entry cp775_table [];
+extern unsigned int cp775_table_len;
+extern struct encoding_table_entry cp852_table [];
+extern unsigned int cp852_table_len;
+extern struct encoding_table_entry cp857_table [];
+extern unsigned int cp857_table_len;
+extern struct encoding_table_entry cp858_table [];
+extern unsigned int cp858_table_len;
+extern struct encoding_table_entry cp853_table [];
+extern unsigned int cp853_table_len;
+extern struct encoding_table_entry cp860_table [];
+extern unsigned int cp860_table_len;
+extern struct encoding_table_entry cp861_table [];
+extern unsigned int cp861_table_len;
+extern struct encoding_table_entry cp863_table [];
+extern unsigned int cp863_table_len;
+extern struct encoding_table_entry cp865_table [];
+extern unsigned int cp865_table_len;
 #ifdef use_cjk_tables
 extern struct encoding_table_entry big5_table [];
 extern unsigned int big5_table_len;
