@@ -74,7 +74,7 @@ static struct prefixspec prefixmap [] =
 	{F5, 0, "diaeresis (umlaut)/dialytika", "¨", "\"x", "x:"},
 	{F6, altshift_mask, "dot above/dasia", "˙/῾", "x."},
 	{F6, ctrlshift_mask, "macron/descender", "¯", "x-"},
-	{F6, alt_mask, "caron (háček)/psili", " ̌/᾿", "x<", "x,,"},
+	{F6, alt_mask, "háček (caron)/psili", " ̌/᾿", "x<", "x,,"},
 	{F6, shift_mask, "grave/varia", "`", "`x", "x!"},
 	{F6, ctrl_mask, "circumflex/oxia", "^", "^x", "x>", "x´"},
 	{F6, 0, "acute (d'aigu)/tonos", "´", "'x", "x´", "x'"},
@@ -100,7 +100,7 @@ static struct prefixspec prefixmap [] =
 	{COMPOSE, '/', "stroke", "/", "x/"},
 	{COMPOSE, '-', "macron/descender", "¯", "x-"},
 	{COMPOSE, 0x00AF /*¯*/, "macron/descender", "¯", "x-"},
-	{COMPOSE, '<', "caron (háček)/psili", " ̌/᾿", "x<", "x,,"},
+	{COMPOSE, '<', "háček (caron)/psili", " ̌/᾿", "x<", "x,,"},
 	{COMPOSE, '.', "dot above/dasia", "˙/῾", "x."},
 	{COMPOSE, 0x02D9 /*˙*/, "dot above/dasia", "˙/῾", "x."},
 	{COMPOSE, '(', "breve/vrachy", "˘", "x("},
@@ -218,6 +218,34 @@ struct {
 };
 
 
+/**
+   Return all mnemos for a character, concatenated in a string with separators.
+ */
+char *
+mnemos (ucs)
+  int ucs;
+{
+  static char mnemos [MAX_CHARS];
+  char * mpoi = mnemos;
+  int i = 0;
+  strcpy (mpoi, "");
+  mpoi += 0;
+  while (unimnemo3 [i].mnemo != NIL_PTR) {
+	if (unimnemo3 [i].uni == ucs) {
+		int l = strlen (unimnemo3 [i].mnemo);
+		if (mpoi + 1 + l < & mnemos [MAX_CHARS - 1]) {
+			strcpy (mpoi, " ");
+			mpoi += 1;
+			strcpy (mpoi, unimnemo3 [i].mnemo);
+			mpoi += l;
+		}
+	}
+	i ++;
+  }
+  return mnemos;
+}
+
+
 #define dont_debug_compose_mnemonic
 #define dont_debug_compose_patterns
 
@@ -387,14 +415,18 @@ compose (c1, c2)
 
 /**
    Compose character from mnemonic.
+   Param variable_length tweaks preferences for two-letter mnemonic input;
+	variable_length = False: two-letter mnemonic
    Return CHAR_UNKNOWN if mnemonic is unknown.
    Return CHAR_INVALID if character found is invalid (CJK conversion).
  */
+static
 unsigned long
-compose_mnemonic (mnemo)
+compose_mnemonic_mode (variable_length, mnemo)
+  FLAG variable_length;
   char * mnemo;
 {
-  unsigned long ret = compose_mnemo (mnemo, True, False);
+  unsigned long ret = compose_mnemo (mnemo, variable_length, False);
 
   if (no_char (ret)) {
 	/* semi-normalize character sequence: bring letter to front, 
@@ -460,12 +492,24 @@ compose_mnemonic (mnemo)
 		}
 	}
 
-	return compose_mnemo (mbuf, True, False);
+	return compose_mnemo (mbuf, variable_length, False);
   }
 
   return ret;
 }
 
+/**
+   Compose character from mnemonic.
+   Return CHAR_UNKNOWN if mnemonic is unknown.
+   Return CHAR_INVALID if character found is invalid (CJK conversion).
+ */
+unsigned long
+compose_mnemonic (mnemo)
+  char * mnemo;
+{
+  return compose_mnemonic_mode (True, mnemo);
+}
+
 static
 void
 postpat (ps, patpoia, patpoib)
@@ -513,7 +557,7 @@ compose_three_patterns (base, pat1a, pat1b, pat2a, pat2b, pat3a, pat3b)
   if (pat3a) {
 	strcat (mnemo, pat3a);
   }
-  composed = compose_mnemonic (mnemo);
+  composed = compose_mnemonic_mode (False, mnemo);
   if (! no_char (composed)) {
 	return composed;
   } else if (pat1b != NIL_PTR) {
@@ -633,7 +677,7 @@ compose_base_patterns (base, ps, ps2, ps3)
   }
   * mnemopoi = '\0';
 
-  composed = compose_mnemonic (mnemo);
+  composed = compose_mnemonic_mode (False, mnemo);
 
   if (no_char (composed) && ps->pat2 != NIL_PTR) {
 	patpoi = ps->pat2;
@@ -648,7 +692,7 @@ compose_base_patterns (base, ps, ps2, ps3)
 	}
 	* mnemopoi = '\0';
 
-	composed = compose_mnemonic (mnemo);
+	composed = compose_mnemonic_mode (False, mnemo);
   }
 
   if (no_char (composed) && ps->pat3 != NIL_PTR) {
@@ -664,7 +708,7 @@ compose_base_patterns (base, ps, ps2, ps3)
 	}
 	* mnemopoi = '\0';
 
-	composed = compose_mnemonic (mnemo);
+	composed = compose_mnemonic_mode (False, mnemo);
   }
 
   return composed;
@@ -694,7 +738,7 @@ decompose (basepoi)
 				return mp;
 			} else if (basenamelen > 1) {
 				/* "schwa&" -> schwa */
-				unsigned long c = compose_mnemo (basename, True, True);
+				unsigned long c = compose_mnemo (basename, False, True);
 				if (! no_char (c)) {
 					* basepoi = c;
 					return mp;
@@ -710,6 +754,14 @@ decompose (basepoi)
 /**
    Compose character (may be precomposed already) 
    from mnemonic patterns and fill-in character.
+   Assume two-letter mnemonic, so param variable_length of compose_mnemo 
+	should be passed as False.
+	This property propagates to
+	- compose_multiple_patterns -> compose_three_patterns
+		-> compose_mnemonic (if called from there);
+	and to
+	compose_base_patterns -> compose_mnemonic (if called from there)
+
    Return CHAR_UNKNOWN if mnemonic is unknown.
    Return CHAR_INVALID if character found is invalid (CJK conversion).
  */
@@ -727,12 +779,12 @@ compose_patterns (base, ps, ps2, ps3)
   }
 
 #ifdef debug_compose_patterns
-	printf ("compose_patterns %04X\n	%s %s %s %s %04X\n", base, ps->accentname, ps->pat1, ps->pat2, ps->pat3);
+	printf ("compose_patterns %04X\n	%s %s %s %s\n", base, ps->accentname, ps->pat1, ps->pat2, ps->pat3);
 	if (ps2) {
-		printf ("	%s %s %s %s %04X\n", ps2->accentname, ps2->pat1, ps2->pat2, ps2->pat3);
+		printf ("	%s %s %s %s\n", ps2->accentname, ps2->pat1, ps2->pat2, ps2->pat3);
 	}
 	if (ps3) {
-		printf ("	%s %s %s %s %04X\n", ps3->accentname, ps3->pat1, ps3->pat2, ps3->pat3);
+		printf ("	%s %s %s %s\n", ps3->accentname, ps3->pat1, ps3->pat2, ps3->pat3);
 	}
 #endif
 
