@@ -28,9 +28,9 @@ extern void DIRECTdosmousegetxy _((void));
 \*======================================================================*/
 
 #ifdef dpmi_call
-__dpmi_regs mregs;
+static __dpmi_regs mregs;
 #else
-union REGS mregs;
+static union REGS mregs;
 #endif
 
 
@@ -131,12 +131,12 @@ mousegetch ()
 void
 DIRECTdosmousegetxy ()
 {
-  mouse_lastbutton = mouse_button;
+  /* remember previous mouse click */
   if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
 	mouse_prevbutton = mouse_button;
+	mouse_prevxpos = mouse_xpos;
+	mouse_prevypos = mouse_ypos;
   }
-  mouse_lastxpos = mouse_xpos;
-  mouse_lastypos = mouse_ypos;
 
   mouse_button = new_button;
 
