@@ -1,7 +1,21 @@
-/* options */
+/* debug options */
+#define dont_debug_mouse
+
 #define dont_dynamic_hilite_tracking
 #define dont_debug_hilite_tracking
-#define dont_debug_mouse
+
+#ifdef debug_hilite_tracking
+# define debug_mouse
+#endif
+
+#ifdef debug_mouse
+#define trace_mouse(tag)	do_trace_mouse (tag)
+extern void do_trace_mouse _((char * tag));
+#else
+#define trace_mouse(tag)	
+#endif
+
+/* options */
 extern void mouse_hilite_mode _((FLAG hilite));
 extern void mouse_hilite_tracking_on _((int startx, int starty, int firstrow, int lastrow));
 extern void mouse_hilite_tracking_off _((void));
@@ -26,7 +40,7 @@ extern int inputreadyafter _((int fid, int msec));
 
 /* terminal encoding detection */
 #ifdef msdos
-extern int get_codepage _((void));
+extern unsigned int get_codepage _((void));
 #endif
 
 /* terminal screen mode handling */
@@ -91,6 +105,7 @@ extern char * scrollbgansi;	/* scrollbar background */
 extern void disp_normal _((void));
 extern void disp_selected _((FLAG bg, FLAG border));
 extern void disp_colour _((int));
+extern void disp_cursor _((FLAG));
 extern void reverse_on _((void));
 extern void reverse_off _((void));
 extern void altcset_on _((void));
@@ -163,13 +178,17 @@ extern unsigned long _readchar_nokeymap _((void));
 extern int read1byte _((void));
 extern int char_ready_within _((int msec, char * debug_tag));
 
-/* mouse handling functions */
+/* mouse handling */
 extern void DIRECTcrttool _((void));
 extern void DIRECTxterm _((void));
 extern void TRACKxterm _((void));
 extern void TRACKxtermT _((void));
-extern FLAG mouse_hilite_tracking;
-extern FLAG mouse_button_event_tracking;
+
+extern mousebutton_type mouse_button, mouse_prevbutton;
+extern int mouse_xpos, mouse_ypos, mouse_prevxpos, mouse_prevypos;
+extern int mouse_shift;		/* modifier status */
+extern FLAG report_release;	/* selectively suppress release event */
+extern FLAG window_focus;	/* does the window have mouse/keyboard focus? */
 
 /* auxiliary menu functions */
 extern void menu_mouse_mode _((FLAG));
@@ -185,12 +204,15 @@ extern FLAG can_reverse_mode;
 extern FLAG can_hide_cursor;
 extern FLAG can_dim;
 extern FLAG can_alt_cset;
+extern FLAG use_mouse_hilite_tracking;
+extern FLAG use_mouse_button_event_tracking;
 
 /* terminal feature usage flags */
 extern FLAG use_ascii_graphics;		/* use ASCII graphics for borders */
 extern FLAG use_vga_block_graphics;	/* charset is VGA with block graphics */
 extern FLAG use_pc_block_graphics;	/* alternate charset is VGA with block graphics */
 extern FLAG use_vt100_block_graphics;
+extern FLAG use_appl_cursor;
 extern FLAG use_appl_keypad;
 extern char menu_border_style;
 extern int menumargin;
@@ -200,6 +222,8 @@ extern FLAG explicit_border_style;
 extern FLAG explicit_scrollbar_style;
 extern FLAG use_script_colour;
 extern FLAG use_mouse;
+extern FLAG use_mouse_anymove_inmenu;
+extern FLAG use_mouse_anymove_always;
 extern FLAG use_bold;
 extern FLAG avoid_reverse_colour;
 extern FLAG use_bgcolor;
