@@ -61,6 +61,14 @@ FLAG hp_shift_mode = False;
 static
 FLAG sco_shift_mode = False;
 
+/* mouse handling */
+mousebutton_type mouse_button, mouse_prevbutton;
+int mouse_xpos, mouse_ypos, mouse_prevxpos, mouse_prevypos;
+int mouse_shift = 0;		/* modifier status */
+FLAG report_release = False;	/* selectively suppress release event */
+FLAG window_focus = True;	/* does the window have mouse/keyboard focus? */
+static int mouse_buttons_pressed = 0;	/* count pressed buttons */
+
 
 /*======================================================================*\
 |*		Local function declaration				*|
@@ -377,8 +385,6 @@ get_digits (result)
 
 #define dont_debug_mapped_keyboard
 
-#define dont_debug_focus
-
 #define max_stamps 10
 static long last_sec = 0;
 static long last_msec = 0;
@@ -464,14 +470,20 @@ readchar ()
   waitingforinput = True;
 
   c = _readchar ();
-  while (command (c) == MOUSEescape
-	&& mouse_button == releasebutton && report_release == False)
+  /* check report_release to suppress subsequent release actions 
+     (esp after menu item selection) 
+     when multiple buttons are pressed simultaneously */
+  while (report_release == False &&
+	 command (c) == MOUSEescape && mouse_button == releasebutton)
   {
-#ifdef debug_focus
-	printf ("ignoring MOUSE release event\n");
+#ifdef debug_mouse
+	printf ("readchar: ignoring MOUSE release (report_release == False)\n");
 #endif
 	c = _readchar ();
   }
+#ifdef debug_mouse
+  printf ("readchar: report_release = False\n");
+#endif
   report_release = False;
 
 #ifdef winchhere
@@ -526,6 +538,9 @@ readcharacter_allbuttons ()
 
   report_winchg = 1;
 
+#ifdef debug_mouse
+  printf ("readchar_allbuttons: report_release = True\n");
+#endif
   report_release = True;
   c = readcharacter ();
 
@@ -568,7 +583,7 @@ readcharacter_mapping (map_keyboard, return_unicode)
 	FLAG skipmouse = False;
 
 	if (command (unichar) == FOCUSout) {
-#ifdef debug_focus
+#ifdef debug_mouse
 		printf ("FOCUS out\n");
 #endif
 		FOCUSout ();
@@ -576,20 +591,20 @@ readcharacter_mapping (map_keyboard, return_unicode)
 		skip_1release = True;
 		skipmouse = True;
 	} else if (command (unichar) == FOCUSin) {
-#ifdef debug_focus
+#ifdef debug_mouse
 		printf ("FOCUS in\n");
 #endif
 		FOCUSin ();
 		skipmouse = True;
 	} else if (command (unichar) == MOUSEescape) {
 		if (mouse_button == releasebutton && skip_1release) {
-#ifdef debug_focus
+#ifdef debug_mouse
 			printf ("skipping 1 MOUSE release event\n");
 #endif
 			skipmouse = True;
 			skip_1release = False;
 		} else if (mouse_button != releasebutton && skip_1click) {
-#ifdef debug_focus
+#ifdef debug_mouse
 			printf ("skipping 1 MOUSE event\n");
 #endif
 			skipmouse = True;
@@ -597,7 +612,7 @@ readcharacter_mapping (map_keyboard, return_unicode)
 			skip_1release = False;
 		} else if (! window_focus) {
 			/* should not be needed anymore */
-#ifdef debug_focus
+#ifdef debug_mouse
 			printf ("skipping MOUSE event while focus out\n");
 #endif
 			skipmouse = True;
@@ -608,7 +623,7 @@ readcharacter_mapping (map_keyboard, return_unicode)
 		return readcharacter_mapping (map_keyboard, return_unicode);
 	} else {
 		/* WINCH exception isn't passing by here... */
-#ifdef debug_focus
+#ifdef debug_mouse
 		if (command (unichar) == MOUSEescape) {
 			if (mouse_button == releasebutton) {
 				printf ("passing MOUSE release event\n");
@@ -754,6 +769,60 @@ readcharacter_unicode_mapped ()
 
 /*-------------------------------------------------------------------------*/
 
+#ifdef debug_mouse
+
+void
+do_trace_mouse (tag)
+  char * tag;
+{
+  static char * button_name [] = {
+	"releasebutton",
+	"leftbutton",
+	"middlebutton",
+	"rightbutton",
+	"movebutton",
+	"wheelup",
+	"wheeldown"};
+
+  printf ("[%s] ", tag);
+  if (mouse_button != releasebutton) {
+	printf ("%s", button_name [mouse_button]);
+  } else {
+	printf ("release (%s)", button_name [mouse_prevbutton]);
+  }
+  printf (" @ %d:%d (%d:%d)\n", mouse_ypos, mouse_xpos, mouse_prevypos, mouse_prevxpos);
+}
+
+#endif
+
+static
+void
+notice_previous_click ()
+{
+  /* remember previous mouse click */
+  if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
+	mouse_prevbutton = mouse_button;
+	mouse_prevxpos = mouse_xpos;
+	mouse_prevypos = mouse_ypos;
+	mouse_buttons_pressed ++;
+  }
+}
+
+static
+void
+fix_mouse_release_event ()
+{
+  /* workaround for incorrect mouse wheel escape sequences */
+  if (mouse_button == releasebutton) {
+	if (mouse_buttons_pressed > 0) {
+		mouse_buttons_pressed --;
+	} else {
+		/* fix the event report */
+		mouse_button = movebutton;
+	}
+  }
+}
+
 /*
  * DIRECT...getxy () reads in the information part of a cursor escape sequence
  */
@@ -766,12 +835,7 @@ DIRECTxtermgetxy (code)
   int xpos;
   int ypos;
 
-  mouse_lastbutton = mouse_button;
-  if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
-	mouse_prevbutton = mouse_button;
-  }
-  mouse_lastxpos = mouse_xpos;
-  mouse_lastypos = mouse_ypos;
+  notice_previous_click ();
 
   if (code == 't') {
 	mouse_button = releasebutton;
@@ -787,6 +851,11 @@ DIRECTxtermgetxy (code)
 	if (button == quit_char) {
 		quit = True;
 	}
+	if (bidi_screen && button == '$') {
+		/* fix mlterm quirk for mouse wheel scroll down */
+		button = 'a';
+		button = '#';
+	}
 	button -= 32;
 	mouse_shift = button & 0x1C;
 	if (button & 32) {
@@ -812,6 +881,8 @@ DIRECTxtermgetxy (code)
 	}
   }
 
+  fix_mouse_release_event ();
+
   xpos = _readchar_nokeymap ();
   if (xpos == quit_char) {
 	quit = True;
@@ -823,32 +894,17 @@ DIRECTxtermgetxy (code)
   }
   mouse_ypos = ypos - 33 - MENU;
 
-#ifdef debug_hilite_tracking
+#ifdef debug_mouse
   printf ("mouse %c: ", code);
-  {
-	static char * button_name [] = {
-		"releasebutton",
-		"leftbutton",
-		"middlebutton",
-		"rightbutton",
-		"movebutton",
-		"wheelup",
-		"wheeldown"};
-	if (mouse_button != releasebutton) {
-		printf ("%s", button_name [mouse_button]);
-	} else {
-		printf ("release (%s)", button_name [mouse_prevbutton]);
-	}
-  }
-  printf (" @ %d, %d\n", mouse_xpos, mouse_ypos);
+  trace_mouse ("DIRECTxtermgetxy");
 #endif
 
 #ifdef unix
-  if (mouse_button_event_tracking && mouse_button == releasebutton) {
+  if (use_mouse_button_event_tracking && mouse_button == releasebutton) {
 	mouse_button_event_mode (False);
   }
 
-  if (mouse_hilite_tracking && mouse_button == leftbutton) {
+  if (use_mouse_hilite_tracking && mouse_button == leftbutton) {
 	/* handle hilite tracking */
 	if (mouse_ypos < 0 || mouse_ypos > last_y 
 	    || (mouse_xpos == XMAX && disp_scrollbar) 
@@ -884,12 +940,7 @@ DIRECTcrttoolgetxy ()
   int xpos;
   int ypos;
 
-  mouse_lastbutton = mouse_button;
-  if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
-	mouse_prevbutton = mouse_button;
-  }
-  mouse_lastxpos = mouse_xpos;
-  mouse_lastypos = mouse_ypos;
+  notice_previous_click ();
 
   c = get_digits (& ypos); /* c should be ';' */
   c = get_digits (& xpos);
@@ -903,6 +954,8 @@ DIRECTcrttoolgetxy ()
 	mouse_button = releasebutton;
   }
 
+  fix_mouse_release_event ();
+
   mouse_ypos = ypos - 1 - MENU;
   mouse_xpos = xpos - 1;
 }
@@ -916,6 +969,9 @@ DIRECTcrttoolgetxy ()
 void
 DIRECTxterm ()
 {
+#ifdef debug_mouse
+  printf ("DIRECTxterm\n");
+#endif
   DIRECTxtermgetxy ('M');
   MOUSEescape ();
 }
@@ -923,6 +979,9 @@ DIRECTxterm ()
 void
 TRACKxterm ()
 {
+#ifdef debug_mouse
+  printf ("TRACKxterm\n");
+#endif
   DIRECTxtermgetxy ('t');
   MOUSEescape ();
 }
@@ -930,6 +989,9 @@ TRACKxterm ()
 void
 TRACKxtermT ()
 {
+#ifdef debug_mouse
+  printf ("TRACKxtermT\n");
+#endif
   DIRECTxtermgetxy ('T');
   MOUSEescape ();
 }
@@ -960,12 +1022,7 @@ CURSgetxy ()
 
   int button = 0;
 
-  mouse_lastbutton = mouse_button;
-  if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
-	mouse_prevbutton = mouse_button;
-  }
-  mouse_lastxpos = mouse_xpos;
-  mouse_lastypos = mouse_ypos;
+  notice_previous_click ();
 
   request_mouse_pos ();
 
@@ -997,6 +1054,8 @@ CURSgetxy ()
 	mouse_button = releasebutton;
   }
 
+  fix_mouse_release_event ();
+
   mouse_xpos = mouse_x;
   mouse_ypos = mouse_y - MENU;
 }
@@ -1018,12 +1077,7 @@ CURSgetxy ()
 # define mouse_y Mouse_status.y
 #endif
 
-  mouse_lastbutton = mouse_button;
-  if (mouse_button == leftbutton || mouse_button == rightbutton || mouse_button == middlebutton) {
-	mouse_prevbutton = mouse_button;
-  }
-  mouse_lastxpos = mouse_xpos;
-  mouse_lastypos = mouse_ypos;
+  notice_previous_click ();
 
 #ifdef NCURSES_VERSION
   getmouse (& mouse_info);
@@ -1047,6 +1101,8 @@ CURSgetxy ()
 	mouse_button = releasebutton;
   }
 
+  fix_mouse_release_event ();
+
   mouse_xpos = mouse_x - 1;
   mouse_ypos = mouse_y - 1 - MENU;
 }
@@ -2054,14 +2110,14 @@ check_ctrl_byte:
 		if (ch == ';' && second_esc_char == '[') {
 			ch = read1byte ();
 			if (ch >= '2' && ch <= '9') {
-				/* ESC [1;2A -> ESC [1A + shift etc */
+				/* ESC [1;2A → ESC [1A + shift etc */
 				/* special: map '9' to applkeypad_mask */
 				keyshift |= (ch & ~'0') - 1;
 			} else if (ch == '1') {
 				ch = read1byte ();
 				if (ch >= '0' && ch <= '6') {
 					/* xterm with alwaysUseMods: */
-					/* ESC [1;11A -> ESC [1A + alt etc */
+					/* ESC [1;11A → ESC [1A + alt etc */
 					keyshift |= applkeypad_mask | ((ch & ~'0') + 1);
 				} else {
 					q_put (';');
@@ -2079,7 +2135,7 @@ check_ctrl_byte:
 		} else if (ch == ';' && second_esc_char == '1') {
 			ch = read1byte ();
 			if (ch >= '2' && ch <= '8') {
-				/* xterm/VT52 mode: ESC 1;2A -> ESC 1A + shift etc */
+				/* xterm/VT52 mode: ESC 1;2A → ESC 1A + shift etc */
 				keyshift |= (ch & ~'0') - 1;
 			} else {
 				q_put (';');
@@ -2093,15 +2149,15 @@ check_ctrl_byte:
 			   && (second_esc_char == 'O' || 
 			       (hp_shift_mode && ! second_esc_char))) {
 			if (ch >= '2' && ch <= '9') {
-				/* ESC O2A -> ESC OA + shift etc */
-				/* HP: ESC 2p -> ESC p + shift etc */
+				/* ESC O2A → ESC OA + shift etc */
+				/* HP: ESC 2p → ESC p + shift etc */
 				/* special: map '9' to applkeypad_mask */
 				keyshift |= (ch & ~'0') - 1;
 			} else if (ch == '1') {
 				ch = read1byte ();
 				if (ch >= '0' && ch <= '6') {
-					/* ESC O11P -> ESC OP + alt etc */
-					/* HP: ESC 11p -> ESC p + alt etc */
+					/* ESC O11P → ESC OP + alt etc */
+					/* HP: ESC 11p → ESC p + alt etc */
 					keyshift |= applkeypad_mask | ((ch & ~'0') + 1);
 				} else {
 					q_put ('1');
@@ -2119,7 +2175,7 @@ check_ctrl_byte:
 			char newkeyshift;
 			char newch = '\0';
 			if (ch >= '2' && ch <= '9') {
-				/* SCO: ESC [2M -> ESC [M + shift etc */
+				/* SCO: ESC [2M → ESC [M + shift etc */
 				/* special: map '9' to applkeypad_mask */
 				newkeyshift = (ch & ~'0') - 1;
 				newch = read1byte ();
@@ -2137,7 +2193,7 @@ check_ctrl_byte:
 				printf ("-> <^[%s> + 1/%02X\n", ctrl_queue + 1, ch);
 #endif
 				if (ch >= '0' && ch <= '6') {
-					/* SCO: ESC [11M -> ESC [M + alt etc */
+					/* SCO: ESC [11M → ESC [M + alt etc */
 					newkeyshift = applkeypad_mask | ((ch & ~'0') + 1);
 					newch = read1byte ();
 					if (newch >= 'A' && newch <= 'Z') {
@@ -2147,7 +2203,7 @@ check_ctrl_byte:
 						q_put (ch);
 					}
 				} else if (ch == ';') {
-					/* xterm SCO: ESC [1;5R -> ESC [R + alt etc */
+					/* xterm SCO: ESC [1;5R → ESC [R + alt etc */
 					ch = read1byte ();
 					keyshift |= (ch & ~'0') - 1;
 					newch = read1byte ();
@@ -2180,25 +2236,25 @@ check_ctrl_byte:
 			}
 		} else if (prev_ch >= '0' && prev_ch <= '9') {
 			if (ch == '^') {
-				/* ESC [5^ -> ESC [5~ + control (rxvt) */
+				/* ESC [5^ → ESC [5~ + control (rxvt) */
 				keyshift |= ctrl_mask;
 				ch = '~';
 			} else if (ch == '$') {
-				/* ESC [5$ -> ESC [5~ + shift (rxvt) */
+				/* ESC [5$ → ESC [5~ + shift (rxvt) */
 				keyshift |= shift_mask;
 				ch = '~';
 			} else if (ch == '@') {
-				/* ESC [5@ -> ESC [5~ + control+shift (rxvt) */
+				/* ESC [5@ → ESC [5~ + control+shift (rxvt) */
 				keyshift |= ctrlshift_mask;
 				ch = '~';
 			}
 		} else if (ch >= 'a' && ch <= 'd') {
 			if (prev_ch == 'O') {
-				/* ESC Oa -> ESC OA + control (rxvt) */
+				/* ESC Oa → ESC OA + control (rxvt) */
 				keyshift |= ctrl_mask;
 				ch -= 'a' - 'A';
 			} else if (prev_ch == '[') {
-				/* ESC [a -> ESC [A + shift (rxvt) */
+				/* ESC [a → ESC [A + shift (rxvt) */
 				keyshift |= shift_mask;
 				ch -= 'a' - 'A';
 			}
@@ -2384,19 +2440,19 @@ check_ctrl_byte:
 
 		if (keyproc == DIRECTxterm) {
 #ifdef debug_mouse
-			printf ("xterm mouse\n");
+			printf ("scanned xterm mouse escape\n");
 #endif
 			DIRECTxtermgetxy ('M');
 			keyproc = MOUSEescape;
 		} else if (keyproc == TRACKxterm) {
 #ifdef debug_mouse
-			printf ("xterm mouse tracking\n");
+			printf ("scanned xterm mouse tracking escape\n");
 #endif
 			DIRECTxtermgetxy ('t');
 			keyproc = MOUSEescape;
 		} else if (keyproc == TRACKxtermT) {
 #ifdef debug_mouse
-			printf ("xterm mouse tracking\n");
+			printf ("scanned xterm mouse tracking escape\n");
 #endif
 			DIRECTxtermgetxy ('T');
 			keyproc = MOUSEescape;
