@@ -520,10 +520,6 @@ struct fkeyentry fkeymap [] =
 	{"\033[200~",	I},	/* begin bracketed paste */
 	{"\033[201~",	I},	/* end bracketed paste */
 
-	/* SiC! keypad redefinitions */
-	{"\033[57~",	SNL},	/* KP_Enter	*/
-	{"\033[58~",	INSkey, ctrl_mask},	/* control-Ins	*/
-
 	/* Deprecated; mined keyboard add-ons: shifted digits etc*/
 	{"\033[00~",	key_0},
 	{"\033[01~",	key_1},
@@ -537,13 +533,16 @@ struct fkeyentry fkeymap [] =
 	{"\033[09~",	key_9},
 	{"\033[~",	DPC},	/* modified BackSpace (ctrl/shift) */
 
-	/* various modified keypad keys (non-application mode) */
-	{"\033\r",	Popmark},	/* alt-Enter	*/
-	{"\033\n",	Popmark},	/* alt-Enter	*/
+	/* various modified keypad keys (non-application mode),
+	   e.g. with xterm *VT100*modifyCursorKeys: 1 or 0;
+	   not complete (wrt modifiers);
+	   would be detected generically with TERM=xterm-sco
+	 */
 	{"\033[2H",	smallHOMEkey, shift_mask},	/* shift-Pos1	*/
 	{"\033[2F",	smallENDkey, shift_mask},	/* shift-Ende	*/
 	{"\033[5H",	smallHOMEkey, ctrl_mask},	/* control-Pos1	*/
 	{"\033[5F",	smallENDkey, ctrl_mask},	/* control-Ende	*/
+
 	{"\0332A",	MUP, shift_mask},	/* shift-Up	*/
 	{"\0332B",	MDN, shift_mask},	/* shift-Down	*/
 	{"\0332C",	MRT, shift_mask},	/* shift-Right	*/
@@ -557,6 +556,10 @@ struct fkeyentry fkeymap [] =
 	{"\0336C",	MRT, ctrlshift_mask},	/* control-shift-Right	*/
 	{"\0336D",	MLF, ctrlshift_mask},	/* control-shift-Left	*/
 
+	/* mined shortcuts */
+	{"\033\r",	Popmark},	/* alt-Enter	*/
+	{"\033\n",	Popmark},	/* alt-Enter	*/
+
 	/* vim mouse wheel mappings for xterm */
 	{"\033[64~",	PU},	/* shift-wheel-up */
 	{"\033[65~",	PD},	/* shift-wheel-down */
@@ -587,7 +590,7 @@ struct fkeyentry fkeymap [] =
 	{"\033[25~",	F3, shift_mask},	/* F13	*/
 	{"\033[26~",	F4, shift_mask},	/* F14	*/
 	{"\033[28~",	F5, shift_mask},	/* VT220: HELP (%1) */
-	{"\033[29~",	HOP},			/* VT220: DO (Redo %0), xterm: Menu */
+	{"\033[29~",	HOP},		/* VT220: DO (%0 redo), xterm: Menu */
 	{"\033[31~",	F7, shift_mask},	/* F17	*/
 	{"\033[32~",	F8, shift_mask},	/* F18	*/
 	{"\033[33~",	F9, shift_mask},	/* F19	*/
@@ -662,6 +665,28 @@ struct fkeyentry fkeymap [] =
 	{"\033On",	DELkey},	/* Del (kD), keypad .	*/
 	{"\033Ol",	DELkey},	/* Del, keypad ,	*/
 
+	/* MinTTY generic patterns for modified application mode keys */
+	{"\033[1p",	INSkey},	/* Ins (kI), keypad 0	*/
+	{"\033[1q",	ENDkey},	/* End (K4), keypad 1	*/
+	{"\033[1r",	MDN},	/* dn (kd), keypad 2	*/
+	{"\033[1s",	PD},	/* PgDn (K5), keypad 3	*/
+	{"\033[1t",	MLF},	/* lf (kl), keypad 4	*/
+	{"\033[1u",	HOP},	/* mid (K2), keypad 5	*/
+	{"\033[1v",	MRT},	/* rt (kr), keypad 6	*/
+	{"\033[1w",	HOMEkey},	/* Pos1 (K1), keypad 7	*/
+	{"\033[1x",	MUP},	/* up (ku), keypad 8	*/
+	{"\033[1y",	PU},	/* PgUp (K3), keypad 9	*/
+	{"\033[1n",	DELkey},	/* Del (kD), keypad .	*/
+	{"\033[1l",	DELkey},	/* Del, keypad ,	*/
+	{"\033[1M",	SNL},	/* Enter (@8) */
+	{"\033[1o",	FIND},	/* keypad /	*/
+	{"\033[1j",	I},	/* keypad *	*/
+	{"\033[1m",	SU},	/* keypad -	*/
+	{"\033[1k",	SD},	/* keypad +	*/
+
+	/* MinTTY generic patterns for modified TAB */
+	{"\033[1Z",	STAB},	/* TAB	*/
+
 	/* Sun */
 	{"\033[208z",	F6, shift_mask},	/* R1 -> grave */
 	{"\033[209z",	F6, ctrl_mask},		/* R2 -> circumflex */
@@ -757,7 +782,7 @@ struct fkeyentry fkeymap [] =
 	/* rxvt */
 	{"\033[7~",	smallHOMEkey},	/* 	*/
 	{"\033[8~",	smallENDkey},	/* 	*/
-	{"\033[Z",	MPW, shift_mask},	/* shift-TAB */
+	{"\033[Z",	MPW},	/* shift-TAB, also MinTTY */
 
 	/* SCO ansi */
 	{"\033[L",	INSkey},	/* (termcap: insert character) */
@@ -769,6 +794,10 @@ struct fkeyentry fkeymap [] =
 	{"\033[O",	FOCUSout},	/* window loses focus */
 	{"\033[I",	FOCUSin},	/* window gains focus */
 
+	/* MinTTY ambiguous width change report */
+	{"\033[1W",	AMBIGnarrow},	/* ambig. width chars are narrow */
+	{"\033[2W",	AMBIGwide},	/* ambig. width chars are wide */
+
 	{NIL_PTR}
 };
 
