@@ -28,7 +28,7 @@
 struct keymapping keymappingtable [] =
 {
 #ifdef use_keymap_tables
-#include "keymapsk.h"
+#include "keymapsk.t"
 #else
 # ifdef __TURBOC__
 	{0, "--"},
