@@ -320,10 +320,15 @@ typedef struct Line LINE;
  * For casting functions with int/void results
  */
 typedef void (* voidfunc) ();
-typedef void (* signalfunc) ();	/* (int) */
-typedef unsigned long (* charfunc) ();
 typedef int (* intfunc) ();
-typedef int (* flagfunc) ();	/* (menuitemtype *, int) */
+#ifdef DEFPROTO_functions
+/* -> weird syntax error... */
+typedef void (* signalfunc) (int);
+typedef int (* flagfunc) (menuitemtype *, int);
+#else
+typedef void (* signalfunc) ();
+typedef int (* flagfunc) ();
+#endif
 
 /* NULL definitions */
 #define NIL_PTR		((char *) 0)
@@ -878,13 +883,7 @@ typedef struct {
 	char * extratag;
 	} menuitemtype;
 extern void separator _((void));
-extern mousebutton_type mouse_button, mouse_lastbutton, mouse_prevbutton;
-extern FLAG window_focus;	/* does the window have mouse/keyboard focus? */
-extern int mouse_shift;
-extern int mouse_xpos, mouse_ypos, mouse_lastxpos, mouse_lastypos;
-extern FLAG report_release;
 extern FLAG flags_changed;	/* Should flag menu area be redrawn? */
-extern FLAG in_menu_border;
 /* from mousemen.c */
 extern void fill_menuitem _((menuitemtype * item, char * s));
 extern void openmenuat _((int col));
@@ -922,6 +921,7 @@ extern void QUICKMENU _((void)), FILEMENU _((void)), EDITMENU _((void));
 extern void SEARCHMENU _((void)), PARAMENU _((void)), OPTIONSMENU _((void));
 extern void handleFlagmenus _((void));
 extern void MOUSEescape _((void)), FOCUSout _((void)), FOCUSin _((void));
+extern void AMBIGnarrow _((void)), AMBIGwide _((void));
 extern void ANSIseq _((void));
 extern void MUP _((void)), MDN _((void)), MLF _((void)), MRT _((void));
 extern void MNW _((void)), MPW _((void));
