@@ -2316,7 +2316,7 @@ local void
 {
 	int flags_gap;
 
-	If cjk_width_data_version
+	If cjk_width_data_version && (! use_vt100_block_graphics || xterm_version > 0)
 	Then	horizontal_bar_width = 2;
 	Fi
 
@@ -3205,10 +3205,12 @@ global int
 	previous_menuline = last_menuline;
 	previous_scroll = last_scroll;
 
-	stayinmenu: continue_menu = True;
+stayinmenu:
+	continue_menu = True;
 
 	/* initial display */
 	display_menu (fullmenu, menulen, menu_width, startcolumn, startline, title, is_flag_menu, disp_only, minwidth, scrolloffset, maxscrolloffset);
+	/* this also sets menu_mouse_mode (True); */
 	If itemno > 0
 	Then	popupmenselected (menu, menu_width, True, startcolumn, startline, itemno - 1, select_index, is_flag_menu);
 	Fi
@@ -3225,22 +3227,10 @@ global int
 	    flush ();
 	    c = readcharacter_allbuttons ();
 
-#define dont_debug_buttons
-#ifdef debug_buttons
+#ifdef debug_mouse
 	    If command (c) == MOUSEescape
 	    Then
-		local char * button_name [] = {
-			"releasebutton",
-			"leftbutton",
-			"middlebutton",
-			"rightbutton",
-			"movebutton",
-			"wheelup",
-			"wheeldown"};
-		If mouse_button != releasebutton
-		Then	printf ("%s\n", button_name [mouse_button]);
-		Else	printf ("release (%s)\n", button_name [mouse_prevbutton]);
-		Fi
+		trace_mouse ("popup_menu");
 	    Fi
 #endif
 
@@ -3251,6 +3241,7 @@ global int
 		popupmenu_active = False;
 		RDwin ();
 		clear_menus ();
+		menu_mouse_mode (False);
 		return ret;
 #else
 		int fullitemno = itemno + scrolloffset;
