@@ -505,10 +505,6 @@ putnarrowutf (utfmarker)
 }
 
 
-#define indicate_illegal_UTF indicate_marked
-#define indicate_narrow indicate_marked
-#define indicate_marked(c) indicate_uni(c, False)
-
 /**
    Indicate highlighted special character property (e.g. illegal code).
  */
@@ -552,6 +548,19 @@ indicate_special (c)
   }
 }
 
+#define dont_debug_indicate
+#ifdef debug_indicate
+#define trace_indicate	printf ("%d\n", __LINE__); 
+#define indicate_special(c)	printf ("%d\n", __LINE__); indicate_special (c)
+#else
+#define trace_indicate	
+#endif
+
+#define indicate_illegal_UTF 	trace_indicate	indicate_marked
+#define indicate_narrow 	trace_indicate	indicate_marked
+#define indicate_marked(c) 	trace_indicate	indicate_uni(c, False)
+
+
 /*
  * put mark character, possibly switching alternate character set
  * putmarkmode and endmarkmode are only called by putmark and for 
@@ -1125,7 +1134,7 @@ put_unichar (unichar)
 	if (iswide (unichar)) {
 		indicate_illegal_UTF (' ');
 	}
-  } else if (suppress_non_BMP && unichar >= 0x10000) {
+  } else if (suppress_non_BMP == True /* sic! */ && unichar >= 0x10000) {
 	indicate_illegal_UTF (REPLACEMENT_CHARACTER);
 	if (iswide (unichar)) {
 		indicate_illegal_UTF (' ');
@@ -1223,7 +1232,8 @@ put_unichar (unichar)
 	/* character cannot be displayed on terminal;
 	   display indication or substitution */
 	if (iscombining_unichar (unichar)) {
-		indicate_narrow ('\'');
+		/*indicate_narrow ('\'');*/
+		indicate_narrow ('`');
 	} else {
 		if (unichar == 0x20AC) {
 			/* EURO SIGN */
@@ -1231,17 +1241,28 @@ put_unichar (unichar)
 		} else if (unichar == 0x2713) {
 			/* ✓ */
 			indicate_narrow ('V');
+		} else if (unichar == 0x02BB) {
+			/* glottal stop ʻokina */
+			indicate_narrow ('\'');
 		} else if (lookup (unichar, list_Quotation_Mark, arrlen (list_Quotation_Mark))) {
 			if (unichar == 0xAB || unichar == 0x2039) {
 				indicate_narrow ('<');
 			} else if (unichar == 0xBB || unichar == 0x203A) {
 				indicate_narrow ('>');
+			} else if ((unichar >= 0x2018 && unichar <= 0x201B)
+				|| unichar == 0x300C || unichar == 0x300D
+				|| (unichar >= 0xFE00 && unichar != 0xFF02)
+				  )
+				{
+				indicate_narrow ('\'');
 			} else {
 				indicate_narrow ('"');
 			}
 		} else if (lookup (unichar, list_Dash, arrlen (list_Dash))) {
 			if (unichar == 0x301C || unichar == 0x3030) {
 				indicate_narrow ('~');
+			} else if (unichar == 0x2E17 || unichar == 0x30A0) {
+				indicate_narrow ('=');
 			} else {
 				indicate_narrow ('-');
 			}
@@ -2227,7 +2248,10 @@ put_line (ly, line, offset, clear_line, positioning)
 			if (XBREAK - count + 1 > 0) {
 				erase_chars (XBREAK - count + 1);
 			}
-		} else if (can_clear_eol) {
+		} else if (can_clear_eol
+			 /* workaround for mlterm bug: */
+			 && ! joining_screen
+			  ) {
 			if (count <= XBREAK) {
 				clear_eol ();
 				set_scrollbar_dirty (ly);
