@@ -34,6 +34,7 @@ static FLAG html_status = NOT_VALID;	/* Status of html_file */
 static int yank_buf_no = 0;	/* Buffer # for trials and multiple buffers */
 static int max_yank_buf_no = 0;	/* Max Buffer # used */
 
+/* pasted area markers for "Paste previous" function */
 static LINE * pasted_start_line = NIL_LINE;
 static char * pasted_start_textp = NIL_PTR;
 static LINE * pasted_end_line = NIL_LINE;
@@ -66,7 +67,7 @@ static struct {
 	char * text;
 	char * file;
 	int lineno;
-	int col;
+	int pos;
 } mark_stack [markstacklen] = {
 	{NIL_LINE, NIL_PTR, NIL_PTR, -1, -1},
 	{NIL_LINE, NIL_PTR, NIL_PTR, -1, -1},
@@ -385,7 +386,7 @@ yank_text (fd, buf_status,
   chars_saved += chars_written;
   lines_saved += lines_written;
 
-  build_string (text_buffer, "%s %d lines to paste buffer (chars/bytes: %ld/%ld) - Paste with %s/Insert", 
+  build_string (text_buffer, "%s paste buffer: lines:chars(bytes) %d:%ld(%ld) - Paste with %s/Insert", 
 	(remove == DELETE) ?
 		append ? "Cut/appended" : "Cut/moved"
 		: append ? "Appended" : "Copied",
@@ -397,7 +398,7 @@ yank_text (fd, buf_status,
 	: emulation == 'p' ? "^U"	/* pico uncut */
 	: "^P"				/* mined paste */
 	);
-  status_msg (text_buffer);
+  status_uni (text_buffer);
 }
 
 
@@ -433,7 +434,7 @@ paste_line (fd, buffer, len)
   register char buffer [MAX_CHARS];
   int * len;
 {
-  int ret = get_line (fd, buffer, len);
+  int ret = get_line (fd, buffer, len, False);
 
   if (ret == ERRORS) {
 	return ret;
@@ -544,7 +545,7 @@ insert_file (fd, stay_old_pos, from_text_file)
 
 /* Insert this text at the current location */
   len = cur_text - cur_line->text;
-  if (insert (line, cur_text, line_buffer) == ERRORS) {
+  if (insert_text (line, cur_text, line_buffer) == ERRORS) {
 	pasted_end_line = NIL_LINE;
 	return;
   }
@@ -577,7 +578,7 @@ insert_file (fd, stay_old_pos, from_text_file)
 	sleep (2);
   } else if (ret == NO_LINE) {	/* Last line read not ended by a '\n' */
 	line = line->next;
-	if (insert (line, line->text, line_buffer) == ERRORS) {
+	if (insert_text (line, line->text, line_buffer) == ERRORS) {
 		pasted_end_line = NIL_LINE;
 		/* give time to read error msg */
 		sleep (2);
@@ -597,8 +598,7 @@ insert_file (fd, stay_old_pos, from_text_file)
 
 /* Fix the screen */
   if (line_count == 0) {		/* Only one line changed */
-	set_cursor (0, y);
-	line_print (y, line);
+	print_line (y, line);
 
 	move_to (x, y);
 	pasted_start_line = cur_line;
@@ -634,6 +634,7 @@ insert_file (fd, stay_old_pos, from_text_file)
   }
 }
 
+
 /**
    Insert the buffer at the current location.
    PASTE () moves the cursor behind the inserted text.
@@ -660,7 +661,7 @@ paste_buffer (old_pos)
 	if ((fd = yankfile (READ, False)) == ERRORS) {
 		int e = geterrno ();
 		if (e == 0 || e == ENOENT /* cygwin */) {
-			error ("Buffer is empty");
+			status_uni ("Buffer is empty - type F1 k for help on copy/paste");
 		} else {
 			error2 ("Cannot read paste buffer: ", serror ());
 		}
@@ -753,7 +754,7 @@ YANKRING ()
 		paste_buffer (True);
 	}
   } else {
-	error ("Cannot undo previous paste");
+	error ("No previous paste to exchange");
   }
 }
 
@@ -774,7 +775,7 @@ paste_HTML ()
 	error ("HTML paste buffer vanished");
 	return;
   }
-  insert_file (fd, True, False);
+  insert_file (fd, False, False);
 }
 
 /*
@@ -834,7 +835,7 @@ WB ()
   if ((yank_fd = yankfile (READ, False)) == ERRORS) {
 	int e = geterrno ();
 	if (e == 0 || e == ENOENT /* cygwin */) {
-		error ("Buffer is empty");
+		error ("Paste buffer is empty");
 	} else {
 		error2 ("Cannot read paste buffer: ", serror ());
 	}
@@ -843,7 +844,7 @@ WB ()
 
 /* Get file name */
   if (get_filename ((hop_flag > 0) ? "Append buffer to file:"
-			       : "Write buffer to file:", wfile_name) != FINE)
+			       : "Write buffer to file (use with HOP to append):", wfile_name) != FINE)
   {
 	return;
   }
@@ -914,7 +915,7 @@ MARK ()
   } else {
 	mark_line = cur_line;
 	mark_text = cur_text;
-	status_msg ("Mark set");
+	status_uni ("Mark set - type F1 k for help on copy/paste");
   }
 }
 
@@ -1037,13 +1038,11 @@ copied_file_name ()
 void
 Pushmark ()
 {
-	int cur_col = get_cur_col ();
-
 	mark_stack [mark_stack_top].line = cur_line;
 	mark_stack [mark_stack_top].text = cur_text;
 	mark_stack [mark_stack_top].file = copied_file_name ();
 	mark_stack [mark_stack_top].lineno = line_number;
-	mark_stack [mark_stack_top].col = cur_col;
+	mark_stack [mark_stack_top].pos = get_cur_pos ();
 
 	mark_stack_top = (mark_stack_top + 1) % markstacklen;
 	if (mark_stack_top == mark_stack_begin) {
@@ -1107,10 +1106,7 @@ Popmark ()
 			mark_stack [mark_stack_poi].text) == NOT_VALID)
 	{
 		int mark_lineno;
-		int mark_col;
 		LINE * open_line;
-		int cur_column;
-		char * cpoi;
 
 		if (switch_files) {
 			if (save_text_load_file (mark_stack [mark_stack_poi].file) == ERRORS) {
@@ -1119,17 +1115,20 @@ Popmark ()
 		}
 
 		mark_lineno = mark_stack [mark_stack_poi].lineno - 1;
-		mark_col = mark_stack [mark_stack_poi].col;
 		open_line = proceed (header->next, mark_lineno);
 		if (open_line == tail) {
 			EFILE ();
 			error ("Stacked position not present anymore");
 		} else {
-			cur_column = 0;
+			int mark_col = mark_stack [mark_stack_poi].pos;
+			int cur_pos = 0;
+			char * cpoi;
+
 			move_to (0, find_y (open_line));
 			cpoi = cur_line->text;
-			while (* cpoi != '\n' && cur_column < mark_col) {
-				advance_char_scr (& cpoi, & cur_column, cur_line->text);
+			while (* cpoi != '\n' && cur_pos < mark_col) {
+				advance_char (& cpoi);
+				cur_pos ++;
 			}
 			move_address (cpoi, y);
 		}
@@ -1224,13 +1223,17 @@ yank_block (remove, append)
   switch (checkmark (mark_line, mark_text)) {
 	case NOT_VALID :
 		if (remove == DELETE) {
+#ifdef msdos
+			status_uni ("Mark not set for Cut and Paste - type Ctrl-Del to delete char, F1 k for help");
+#else
 			if (mined_keypad) {
-				error ("Mark not set for Cut and Paste - type Alt-Del to delete char, F1 k for help");
+				status_uni ("Mark not set for Cut and Paste - type Alt-Del to delete char, F1 k for help");
 			} else {
-				error ("Mark not set for Cut and Paste - type Ctrl-Del to delete char, F1 k for help");
+				status_uni ("Mark not set for Cut and Paste - type F1 k for help");
 			}
+#endif
 		} else {
-			error ("Mark not set for Copy and Paste");
+			status_uni ("Mark not set for Copy and Paste - type F1 k for help");
 		}
 		return;
 	case SMALLER :
@@ -1247,8 +1250,8 @@ yank_block (remove, append)
 				remove, append);
 		yankie ();
 		break;
-	case SAME :		/* Ignore stupid behaviour */
-		status_msg ("Nothing to save");
+	case SAME :
+		status_uni ("No text selected for Copy or Cut");
 		break;
 	default :
 		error ("Internal mark error");
@@ -1262,7 +1265,7 @@ yank_HTML (remove)
 {
   switch (checkmark (mark_line, mark_text)) {
 	case NOT_VALID :
-		error ("Mark not set");
+		error ("HTML tag selection failed");
 		return;
 	case SMALLER :
 		yank_text (htmlfile (WRITE, False), & html_status, 
@@ -1274,8 +1277,8 @@ yank_HTML (remove)
 				cur_line, cur_text, mark_line, mark_text, 
 				remove, False);
 		break;
-	case SAME :		/* Ignore stupid behaviour */
-		status_msg ("Nothing to save");
+	case SAME :
+		error ("HTML tag selection failed");
 		break;
 	default :
 		error ("Internal mark error");
