@@ -1046,8 +1046,7 @@ finished (program, last_exp)
 	ring_bell ();
 	error ("Cannot allocate memory for search expression");
 	return ERRORS;
-  }
-  else {	/* Copy expression. (expression consists of ints!) */
+  } else {	/* Copy expression. (expression consists of ints!) */
 	bcopy (exp_buffer, program->result.expression, length);
 	return FINE;
   }
@@ -1078,8 +1077,9 @@ print_expression (expr)
   int * expr;
 {
   char s [20];
+  int ei = 0;
 
-  while (1) {
+  while (ei < MAX_EXP_SIZE) {
 	s [0] = '\0';
 	if (* expr & NORMAL)	strcat (s, "-");
 	if (* expr & NEGATE)	strcat (s, "!");
@@ -1090,9 +1090,11 @@ print_expression (expr)
 	if (* expr & DONE)	strcat (s, "DONE");
 	printf ("%s%04X ", s, * expr);
 	if (* expr & DONE) break;
+	if (* expr == 0) break;
 	expr ++;
+	ei ++;
   }
-  printf ("\n");
+  printf (" [exp size %d]\n", ei);
 }
 #else
 #define trace_expression(params)	
