@@ -28,6 +28,13 @@ iscombining_unichar (u)
 }
 
 
+/*======================================================================*\
+	Dummies for width.c
+\*======================================================================*/
+
+FLAG suppress_non_BMP = 0;
+
+
 /*======================================================================*\
 	Checking function
 \*======================================================================*/
